from datetime import timedelta

import factory

from concentrator.smev3_v321.models import ApplicantAnswer
from concentrator.smev3_v321.order.constants import CreateOrderStatusMapper
from kinder.core.declaration.tests import factory_declaration
from kinder.core.declaration_status.enum import DSS
from kinder.core.declaration_status.models import DeclarationStatus
from kinder.core.declaration_status.tests import dss_factory
from kinder.core.dict.models import GroupAgeSubCathegory
from kinder.core.direct.models import DirectStatus
from kinder.core.direct.models import DRS
from kinder.core.direct.tests import factory_direct
from kinder.test.base import BaseTC
from kinder.test.utils import faker


class ApplicantAnswerF(factory.DjangoModelFactory):
    class Meta:
        model = ApplicantAnswer


class CreateOrderRequestTestCase(BaseTC):
    """Тестируем работу сервиса CreateOrderRequest"""

    def setUp(self):
        super().setUp()

        self.declaration = factory_declaration.DeclarationF(
            status=DeclarationStatus.objects.get(code=DSS.DIRECTED),
        )

        self.direct = factory_direct.DirectFactory(
            declaration=self.declaration,
        )

        self.direct.status = DirectStatus.objects.get(code=DRS.REGISTER)

        self.status_log = dss_factory.DeclarationStatusLogF(
            declaration=self.declaration,
            status=DeclarationStatus.objects.get(code=DSS.DIRECTED),
        )

        self.direct_status_log = factory_direct.DirectStatusLogF(
            status=self.direct.status, direct=self.direct)

        self.direct.group.sub_age_cat = GroupAgeSubCathegory.objects.get(
            code=GroupAgeSubCathegory.MIN_CODE
        )

        self.direct.document_details = '123456'

        self.status_log_data = {'status__code': self.status_log.status.code}

        self.status_mapper = CreateOrderStatusMapper(
            self.declaration, self.direct)

        self.declaration.mo.days_for_reject_direct = 15

        self.direct.group.unit.address_full = faker('address')

    def test_order_request_without_applicant_answer(self):
        """
        Проверка кода и статуса при переходе направления в статус
        Направлен в ДОО без ответа заявителя
        """
        actual_result = self.status_mapper[self.status_log_data]
        expected_result = (
            230, 'Согласие с предоставленным местом направлено на '
                 f'рассмотрение в {self.declaration.mo.name}.')
        self.assertEqual(actual_result, expected_result)

    def test_order_request_with_applicant_answer(self):
        """
        Проверка кода и статуса при переходе направления в статус
        Направлен в ДОО при наличии ответа заявителя
        """
        ApplicantAnswerF(direct=self.direct, answer=True)
        actual_result = self.status_mapper[self.status_log_data]

        days_for_reject_direct = self.declaration.mo.days_for_reject_direct

        end_date = (
            self.direct.created + timedelta(days=days_for_reject_direct))

        direct_reject_date = f'до {end_date.strftime("%d.%m.%Y")}'

        expected_result = (
            190,
            f'Вам предоставлено место в {self.direct.group.unit.name} в '
            f'группу {self.direct.group.name} '
            f'({self.direct.group.sub_age_cat.name}) в '
            f'соответствии с {self.direct.document_details}. '
            'Вам необходимо явиться по адресу '
            f'{self.direct.group.unit.address_full}'
            f' {direct_reject_date}.')
        self.assertEqual(actual_result, expected_result)
